#ifndef __TXP__BACKENDS__BACKENDS_H
    #define __TXP__BACKENDS__BACKENDS_H


    #include <string>
    #include <iostream>
    #include <fstream>
    #include <locale>


    extern bool DEBUG_EXPORT;


    class TXPFont
    {
        public:
            std::string Name;
    };

    class TXPFontSettings
    {
        public:
            TXPFont *Font;
            int FontSize;
            bool Bold;
            bool Italic;
    };


    class TXPBackend
    {
        protected:
            std::wstring Author;
            std::wstring Title;
            int NumberOfPages;
            time_t CreationTime;

            float CurrentPageWidth_cm;
            float CurrentPageHeight_cm;
            float CurrentPageResolution_dpi;

            bool BottomUpYAxis;

        public:
            TXPBackend();
            virtual ~TXPBackend();

            virtual void Initialize(int NumberOfPages, time_t CreationTime, std::wstring Author=L"", std::wstring Title=L"");
            virtual void NewPage(int PageNumber, float width_cm, float height_cm, float resolution_dpi);
            virtual void FinishPage();
        protected:
            virtual void PutTextDots(int leftDots, int topDots, const std::wstring& text, TXPFontSettings* font);
            virtual void PutPageLinkDots(int leftDots, int topDots, int widthDots, int heightDots, int PageNumber);
            virtual void PutHyperlinkDots(int leftDots, int topDots, int widthDots, int heightDots, const std::string& url);
            virtual void PutLineDots(int x1Dots, int y1Dots, int x2Dots, int y2Dots);
        public:
            virtual void PutText(float left_cm, float top_cm, const std::wstring& text, TXPFontSettings* font);
            virtual void PutPageLink(float left_cm, float top_cm, float width_cm, float height_cm, int PageNumber);
            virtual void PutHyperlink(float left_cm, float top_cm, float width_cm, float height_cm, const std::string& url);
            virtual void PutLine(float x1_cm, float y1_cm, float x2_cm, float y2_cm);
            virtual int PutBookmark(int PageNumber, const std::wstring& Caption, int ParentHandle = -1);
            virtual void Finish();
    };



    // gigantic thanks to
    // http://stackoverflow.com/questions/13422892
    struct no_separator : std::numpunct<char>
    {
        protected:
            virtual std::string do_grouping() const;
    };

    struct wno_separator : std::numpunct<wchar_t>
    {
        protected:
            virtual std::string do_grouping() const;
    };



    class TXPBackendStream : public TXPBackend
    {
        protected:
            std::ostream* stream;
        public:
            TXPBackendStream();
            ~TXPBackendStream();
    };


    class TXPBackendFile : public TXPBackendStream
    {
        public:
            TXPBackendFile(std::string FileName);
            ~TXPBackendFile();
    };


#endif
