
#include "../../include/backends/backends.h"
using namespace std;


    TXPBackend::TXPBackend()
    {
        BottomUpYAxis = false;
        CurrentPageWidth_cm = 0.0f;
        CurrentPageHeight_cm = 0.0f;
        CurrentPageResolution_dpi = 0.0f;
    }

    TXPBackend::~TXPBackend()
    {
    }

    void TXPBackend::Initialize(int NumberOfPages, time_t CreationTime, std::wstring Author, std::wstring Title)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::Initialize\n";

        this->NumberOfPages = NumberOfPages;
        this->Author = Author;
        this->Title = Title;
        this->CreationTime = CreationTime;
    }

    void TXPBackend::Finish()
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::Finish\n";
    }

    void TXPBackend::NewPage(int PageNumber, float Width, float Height, float resolution_dpi)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::NewPage\n";

        CurrentPageWidth_cm = Width;
        CurrentPageHeight_cm = Height;
        CurrentPageResolution_dpi = resolution_dpi;
    }

    void TXPBackend::FinishPage()
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::FinishPage\n";
    }


    void TXPBackend::PutTextDots(int, int, const std::wstring&, TXPFontSettings*)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutTextDots\n";
    }
    void TXPBackend::PutText(float Left_cm, float Top_cm, const std::wstring& text,
        TXPFontSettings* font)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutText\n";

        int leftDots = Left_cm * (CurrentPageResolution_dpi/2.54);
        int topDots = (BottomUpYAxis ? CurrentPageHeight_cm - Top_cm : Top_cm) * (CurrentPageResolution_dpi/2.54);
        PutTextDots(leftDots, topDots, text, font);
    }


    void TXPBackend::PutLineDots(int, int, int, int)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutLineDots\n";
    }
    void TXPBackend::PutLine(float x1_cm, float y1_cm, float x2_cm, float y2_cm)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutLine\n";

        int x1Dots = x1_cm * (CurrentPageResolution_dpi/2.54);
        int y1Dots = (BottomUpYAxis ? CurrentPageHeight_cm - y1_cm : y1_cm) * (CurrentPageResolution_dpi/2.54);
        int x2Dots = x2_cm * (CurrentPageResolution_dpi/2.54);
        int y2Dots = (BottomUpYAxis ? CurrentPageHeight_cm - y2_cm : y2_cm) * (CurrentPageResolution_dpi/2.54);
        PutLineDots(x1Dots, y1Dots, x2Dots, y2Dots);
    }


    void TXPBackend::PutPageLinkDots(int, int, int, int, int)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutPageLinkDots\n";
    }
    void TXPBackend::PutPageLink(float left_cm, float top_cm, float width_cm, float height_cm, int Page)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutPageLink\n";

        int leftDots = left_cm * (CurrentPageResolution_dpi/2.54);
        int topDots = (BottomUpYAxis ? CurrentPageHeight_cm - top_cm : top_cm) * (CurrentPageResolution_dpi/2.54);
        int widthDots = width_cm * (CurrentPageResolution_dpi/2.54);
        int heightDots = height_cm * (CurrentPageResolution_dpi/2.54);
        PutPageLinkDots(leftDots, topDots, widthDots, heightDots, Page);
    }


    void TXPBackend::PutHyperlinkDots(int, int, int, int, const std::string&)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutHyperlinkDots\n";
    }
    void TXPBackend::PutHyperlink(float left_cm, float top_cm, float width_cm, float height_cm, const std::string& url)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutHyperlink\n";

        int leftDots = left_cm * (CurrentPageResolution_dpi/2.54);
        int topDots = (BottomUpYAxis ? CurrentPageHeight_cm - top_cm : top_cm) * (CurrentPageResolution_dpi/2.54);
        int widthDots = width_cm * (CurrentPageResolution_dpi/2.54);
        int heightDots = height_cm * (CurrentPageResolution_dpi/2.54);
        PutHyperlinkDots(leftDots, topDots, widthDots, heightDots, url);
    }


    int TXPBackend::PutBookmark(int, const std::wstring&, int)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackend::PutBookmark\n";

        return -1;
    }




    // gigantic thanks to
    // http://stackoverflow.com/questions/13422892
    std::string no_separator::do_grouping() const
    {
        return "\000";
    }
    std::string wno_separator::do_grouping() const
    {
        return "\000";
    }


    TXPBackendStream::TXPBackendStream()
        : TXPBackend()
    {
        stream = NULL;
    }

    TXPBackendStream::~TXPBackendStream()
    {
        delete stream;
        stream = NULL;
    }



    TXPBackendFile::TXPBackendFile(std::string FileName)
        : TXPBackendStream()
    {
        //std::locale utf8encoding("en_US.UTF-8");
        //stream = new std::wofstream(FileName.c_str());
        stream = new std::ofstream(FileName.c_str());
        //stream->imbue(std::locale(utf8encoding, new no_separator()));
    }

    TXPBackendFile::~TXPBackendFile()
    {
        std::ofstream* FStream = static_cast<std::ofstream*>(stream);
        if(FStream != NULL)
        {
            FStream->flush();
            FStream->close();
        }
    }








