
#include "../../include/backends/postscript.h"
using namespace std;


    TXPBackendPostscript::TXPBackendPostscript(std::string FileName)
        : TXPBackendFile(FileName)
    {
        BottomUpYAxis = true;
        CurrentPageResolution_dpi = 72.0f;
    }

    TXPBackendPostscript::~TXPBackendPostscript()
    {
    }

    std::string TXPBackendPostscript::ToPsString(const std::wstring& str)
    {
        std::string result = "(";
        for(std::wstring::const_iterator i = str.begin(); i != str.end(); i++)
            if(*i <= 127)
                result += *i;
        result += ")";
        return result;
    }

    void TXPBackendPostscript::Initialize(int NumberOfPages, time_t CreationTime,
        std::wstring Author, std::wstring Title)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackendPostscript::Initialize\n";

        TXPBackendFile::Initialize(NumberOfPages, CreationTime, Author, Title);
        struct tm* t = localtime(&CreationTime);

        (*stream) // << "%!PS-Adobe-2.0\n\n"
                  << "%!PS-Adobe-3.0 EPSF-3.0\n\n"
                  << "%%Creator: LaTeX++\n"
                  << "%%Author: " << ToPsString(Author) << "\n"
                  << "%%Title: " << ToPsString(Title) << "\n"
                  << "%%CreationDate: "
                    << setw(4) << setfill('0') << (t->tm_year+1900) << "-"
                    << setw(2) << setfill('0') << (t->tm_mon+1) << "-"
                    << setw(2) << setfill('0') << (t->tm_mday) << "\n"
                 // << setw(2) << setfill('0') << (t->tm_hour)
                 // << setw(2) << setfill('0') << (t->tm_min)
                 // << setw(2) << setfill('0') << (t->tm_sec)
                  << "%%DocumentData: Clean7Bit\n"
                  << "%%LanguageLevel: 2\n" // 1,2 or 3
                  << "%%Pages: " << NumberOfPages << "\n";

    }

    void TXPBackendPostscript::NewPage(int PageNumber, float width_cm, float height_cm, float)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackendPostscript::NewPage\n";

        TXPBackendFile::NewPage(PageNumber, width_cm, height_cm, 72.0f);
        (*stream) << "%%Page: " << PageNumber << " " << PageNumber << "\n"
                  << "%%PageBoundingBox: 0 0 " << (int)(width_cm*CurrentPageResolution_dpi/2.54)
                      << " " << (int)(height_cm*CurrentPageResolution_dpi/2.54) << "\n";
    }

    void TXPBackendPostscript::FinishPage()
    {
        if(DEBUG_EXPORT) cerr << "TXPBackendPostscript::FinishPage\n";

        (*stream) << "showpage\n";
        TXPBackendFile::FinishPage();
    }

    void TXPBackendPostscript::PutTextDots(int leftDots, int topDots, const std::wstring& text,
        TXPFontSettings* font)
    {
        if(DEBUG_EXPORT) cerr << "TXPBackendPostscript::PutTextDots\n";

        (*stream) << "/" << font->Font->Name.c_str() << " findfont " << font->FontSize << " scalefont setfont ";
        (*stream) << leftDots << " " << topDots << " moveto";

        for(std::wstring::const_iterator i = text.begin(); i != text.end();)
        {
            std::string temp = "";
            while(i != text.end() && *i <= 127)
            {
                switch(*i)
                {
                    case '(': temp += "\\("; break;
                    case ')': temp += "\\)"; break;
                    case '\\': temp += "\\\\"; break;
                    default: temp += *i;
                }
                i++;
            }
            if(temp != "")
                (*stream) << " (" << temp << ") show";
            for(; i != text.end() && *i > 127; i++)
                (*stream) << " /" << GlyphName(*i) << " glyphshow";
        }
        (*stream) << "\n";
    }

    void TXPBackendPostscript::Finish()
    {
        if(DEBUG_EXPORT) cerr << "TXPBackendPostscript::Finish\n";

        (*stream) << "%%EOF\n";
        TXPBackendFile::Finish();
    }
