
#include <iostream>
#include "../include/txp/txp.h"
#include "../include/parser/txp_lexer.h"

#include "../include/middleend.h"
#include "../include/backends/postscript.h"
#include "../include/backends/portabledocumentformat.h"

using namespace std;

#define VERSION "0.3.1"

bool LEXER_DEBUG = false;
bool PARSER_DEBUG = false;
bool DEBUG_EXECUTION = false;
bool DEBUG_EXPORT = false;
bool DEBUG_GENERAL = true;


int main(int argc, char** argv)
{
    int backendSelection = 1; // 0=ps, 1=pdf
    string infile = "";
    string outfile = "";
    istream* in = &cin;
    for(int i = 1; i < argc; i++)
    {
        string sargvi = string(argv[i]);
        for(int j = 0; j < sargvi.size(); j++)
            if('A' <= sargvi[j] && sargvi[j] <= 'Z')
                sargvi[j] += 'a'-'A';

        if(     sargvi ==  "-ps" || sargvi ==  "--ps") backendSelection = 0;
        else if(sargvi == "-pdf" || sargvi == "--pdf") backendSelection = 1;
        else if(sargvi ==  "-dl" || sargvi ==  "--dl") LEXER_DEBUG = true;
        else if(sargvi ==  "-dp" || sargvi ==  "--dp") PARSER_DEBUG = true;
        else if(sargvi ==  "-dx" || sargvi ==  "--dx") DEBUG_EXECUTION = true;
        else if(sargvi ==  "-dg" || sargvi ==  "--dg") DEBUG_EXPORT = true;
        else if(sargvi ==   "-q" || sargvi ==   "--q") LEXER_DEBUG = PARSER_DEBUG = DEBUG_EXECUTION = DEBUG_EXPORT = DEBUG_GENERAL = false;
        else if(sargvi ==   "-v" || sargvi ==   "--v") LEXER_DEBUG = PARSER_DEBUG = DEBUG_EXECUTION = DEBUG_EXPORT = DEBUG_GENERAL = true;
        else if((sargvi ==  "-i" || sargvi ==   "--i") && argc > i+1)
                                                       {infile = argv[i+1]; i++;
                                                       if(outfile == "") outfile = infile; }
        else if((sargvi ==  "-o" || sargvi ==   "--o") && argc > i+1)
                                                       {outfile = argv[i+1]; i++;}
        else if(sargvi ==  "-help" || sargvi ==   "--help") {
            cerr << "This is LaTeX++ Version " << VERSION << endl;
            cerr << "by Viktor \"AlgorithMan\" Engelmann" << endl;
            cerr << "http://www.AlgorithMan.de" << endl << endl;

            cerr << "Usage: " << argv[0] << " { PARAMETERS }" << endl;
            cerr << "    where PARAMETERS can be" << endl;
            cerr << "    -i <infile>    if no <infile> is given, input is read from STDIN." << endl;
            cerr << "    -o <outfile>   required! (without file-extension)" << endl;
            cerr << "    -ps            Select the PostScript Backend" << endl;
            cerr << "    -pdf           Select the Portable Document Format Backend (default)" << endl;
            cerr << "    -dl            Show Debug-Messages from the lexer" << endl;
            cerr << "    -dp            Show Debug-Messages from the parser" << endl;
            cerr << "    -dx            Show Debug-Messages from the execution of the code" << endl;
            cerr << "    -dg            Show Debug-Messages from the backend" << endl;
            cerr << "    -v             Verbose mode, show all Debug-Messages" << endl;
            cerr << "    -q             Quiet mode, show no Debug-Messages" << endl;
            cerr << "    -help          Show these help messages" << endl;

            return 0;
        }
    }



    if(DEBUG_GENERAL)
    {
        cerr << "This is LaTeX++ Version " << VERSION << endl;
        cerr << "by Viktor \"AlgorithMan\" Engelmann" << endl;
        cerr << "http://www.AlgorithMan.de" << endl << endl;
    }



    // ============================================================= Parse Input
    if(DEBUG_GENERAL) { cerr << "  Parsing..."; cerr.flush(); }

    if(infile != "")
        in = new ifstream(infile.c_str());
    TXPLexer lexer(*in);
    TXPElement* txpdocument = NULL;
    yy::TXPParser parser(lexer, &txpdocument);
    parser.parse();
    if(in != &cin)
        delete in;



    // =============================================== Construct Abstract Result
    if(DEBUG_GENERAL) { cerr << "\n  Executing..."; cerr.flush(); }

    //TXPMiddleEnd middleend(21.59f, 27.94f); // Letter
    TXPMiddleEnd middleend(21.0f, 29.7f); // DIN A4
    middleend.Author = L"Viktor Engelmann";
    middleend.Title = L"LaTeX++ Test";
    TXPContext context(&middleend);
    if(txpdocument != NULL)
        txpdocument->Execute(&context);
    delete txpdocument;



    // ================================== Convert Abstract Result to Target File
    if(DEBUG_GENERAL) { cerr << "\n  Exporting to " << (backendSelection==0?"PostScript (ps)...":"Portable Document Format (pdf)..."); cerr.flush(); }

    TXPBackend* backend = NULL;
    if(backendSelection == 0)
        backend = new TXPBackendPostscript(outfile+".ps");
    else if(backendSelection == 1)
        backend = new TXPBackendPortableDocumentFormat(outfile+".pdf");

    if(backend != NULL)
        middleend.ProduceResult(backend);
    delete backend;



    if(DEBUG_GENERAL) { cerr << "\n  Done\n"; cerr.flush(); }
    return 0;
}

