
#include "../include/middleend.h"
using namespace std;

        extern bool DEBUG_EXECUTION;
        extern bool DEBUG_EXPORT;


        AbstractPageContent::AbstractPageContent()
        {
        }
        AbstractPageContent::~AbstractPageContent()
        {
        }
        void AbstractPageContent::WriteToBackend(TXPMiddleEnd*, TXPBackend*)
        {
        }



        AbstractTextArea::AbstractTextArea(float left, float top, const std::wstring& text, TXPFontSettings* font)
            : AbstractPageContent()
        {
            this->left = left;
            this->top = top;
            this->text = text;
            this->font = font;
        }
        AbstractTextArea::~AbstractTextArea()
        {
        }
        void AbstractTextArea::WriteToBackend(TXPMiddleEnd*, TXPBackend* backend)
        {
            if(DEBUG_EXPORT) cerr << "AbstractTextArea::WriteToBackend\n";

            backend->PutText(left, top, text, font);
        }



        AbstractPageLink::AbstractPageLink(float left, float top, float width, float height, const std::wstring& label)
            : AbstractPageContent()
        {
            this->left = left;
            this->top = top;
            this->width = width;
            this->height = height;
            this->label = label;
        }
        AbstractPageLink::~AbstractPageLink()
        {
        }
        void AbstractPageLink::WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend)
        {
            if(DEBUG_EXPORT) cerr << "AbstractPageLink::WriteToBackend\n";

            int pagenumber = middleend->GetPageNumber(label);
            if(pagenumber > 0)
                backend->PutPageLink(left, top, width, height, pagenumber);
        }



        AbstractHyperlink::AbstractHyperlink(float left, float top, float width, float height, const std::wstring& url)
            : AbstractPageLink(left, top, width, height, url)
        {
        }
        AbstractHyperlink::~AbstractHyperlink()
        {
        }
        void AbstractHyperlink::WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend)
        {
            if(DEBUG_EXPORT) cerr << "AbstractHyperLink::WriteToBackend\n";

            std::string utf8_url = WStringToUtf8String(label);
            std::string ascii_url = "";
            static std::string hex = "0123456789ABCDEF";
            for(std::string::iterator i = utf8_url.begin(); i != utf8_url.end(); i++)
            {
                unsigned char c = (unsigned char)(*i);
                if(c <= 127)
                    ascii_url += *i;
                else
                    ascii_url = ((ascii_url + "%") + hex[(c & 0xf0) >> 4]) + hex[c & 0xf];
            }
            backend->PutHyperlink(left, top, width, height, ascii_url);
        }



        AbstractBookmark::AbstractBookmark(AbstractPage* Page, const std::wstring& Caption, int depthRelativeToPrevious)
            : AbstractPageContent()
        {
            this->Page = Page;
            this->Caption = Caption;
            this->depthRelativeToPrevious = depthRelativeToPrevious;
        }
        AbstractBookmark::~AbstractBookmark()
        {
        }
        void AbstractBookmark::WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend)
        {
            if(DEBUG_EXPORT) cerr << "AbstractBookmark::WriteToBackend\n";

            for(int relDepth = depthRelativeToPrevious; relDepth <= 0 && !middleend->BookmarkHandleStack.empty(); relDepth++)
                middleend->BookmarkHandleStack.pop();
            if(middleend->BookmarkHandleStack.empty())
                middleend->BookmarkHandleStack.push(0);
            int bookmarkHandle = backend->PutBookmark(Page->GetPageNumber(), Caption, middleend->BookmarkHandleStack.top());
            middleend->BookmarkHandleStack.push(bookmarkHandle);
        }



        AbstractPage::AbstractPage(int PageNumber, float PageWidth, float PageHeight, float MarginTop, float MarginLeft, float MarginRight, float MarginBottom, float Resolution_dpi)
        {
            this->PageNumber = PageNumber;
            this->PageWidth_cm = PageWidth;
            this->PageHeight_cm = PageHeight;
            this->MarginTop_cm = MarginTop;
            this->MarginLeft_cm = MarginLeft;
            this->MarginRight_cm = MarginRight;
            this->MarginBottom_cm = MarginBottom;
            this->PageResolution_dpi = Resolution_dpi;
        }
        AbstractPage::~AbstractPage()
        {
            for(std::list<AbstractPageContent*>::iterator i = Contents.begin(); i != Contents.end(); i++)
                delete *i;
        }
        void AbstractPage::PutText(float left, float top, const std::wstring& text, TXPFontSettings* font)
        {
            if(DEBUG_EXECUTION) cerr << "AbstractPage::PutText\n";
            Contents.push_back(new AbstractTextArea(left, top, text, font));
        }
        void AbstractPage::PutPageLink(float left, float top, float width, float height, const std::wstring& label)
        {
            if(DEBUG_EXECUTION) cerr << "AbstractPage::PutPageLink\n";
            Contents.push_back(new AbstractPageLink(left, top, width, height, label));
        }
        void AbstractPage::PutHyperlink(float left, float top, float width, float height, const std::wstring& url)
        {
            if(DEBUG_EXECUTION) cerr << "AbstractPage::PutHyperlink\n";
            Contents.push_back(new AbstractHyperlink(left, top, width, height, url));
        }
        void AbstractPage::PutBookmark(int depthRelativeToPrevious, const std::wstring& Caption)
        {
            if(DEBUG_EXECUTION) cerr << "AbstractPage::PutBookmark\n";
            Contents.push_back(new AbstractBookmark(this, Caption, depthRelativeToPrevious));
        }
        void AbstractPage::WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend)
        {
            if(DEBUG_EXECUTION) cerr << "AbstractPage::WriteToBackend\n";
            backend->NewPage(PageNumber, PageWidth_cm, PageHeight_cm, PageResolution_dpi);
            for(std::list<AbstractPageContent*>::iterator i = Contents.begin(); i != Contents.end(); i++)
                (*i)->WriteToBackend(middleend, backend);
            backend->FinishPage();
        }
        int AbstractPage::GetPageNumber()
        {
            return PageNumber;
        }





        TXPMiddleEnd::TXPMiddleEnd(float DefaultWidth_cm, float DefaultHeight_cm, float DefaultResolution_dpi)
        {
            DefaultMarginTop_cm = 2.0f;
            DefaultMarginLeft_cm = 2.0f;
            DefaultMarginRight_cm = 2.0f;
            DefaultMarginBottom_cm = 2.0f;

            this->DefaultWidth_cm = DefaultWidth_cm;
            this->DefaultHeight_cm = DefaultHeight_cm;
            this->DefaultResolution_dpi = DefaultResolution_dpi;

            CurrentPage = NULL;
            NextPageNumber = 1;
            left_cm = 0.0f;
            top_cm = 0.0f;

            DefaultFont = new TXPFontSettings;
            DefaultFont->FontSize = 12;
            DefaultFont->Bold = false;
            DefaultFont->Italic = false;
            DefaultFont->Font = new TXPFont;
            DefaultFont->Font->Name = "Courier";
            //DefaultFont->Font->Name = "/usr/share/fonts/truetype/ttf-dejavu/DejaVuSerif.ttf";

            Author = L"";
            Title = L"";
            CreationTime = time(NULL);
        }

        TXPMiddleEnd::~TXPMiddleEnd()
        {
            for(std::list<AbstractPage*>::iterator i = Pages.begin(); i != Pages.end(); i++)
                delete *i;
            Pages.clear();
            CurrentPage = NULL;
            delete DefaultFont->Font;
            delete DefaultFont;
        }



        void TXPMiddleEnd::AddText(float left, float top, const std::wstring& text, TXPFontSettings* font)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::AddText\n";

            if(font == NULL)
                font = DefaultFont;

            bool isWhiteSpace = true;
            for(std::wstring::const_iterator i = text.begin(); isWhiteSpace && i != text.end(); i++)
                if(*i != ' ' && *i != '\n' && *i != '\t' && *i != '\r')
                    isWhiteSpace = false;

            if(!isWhiteSpace)
            {
                if(CurrentPage == NULL)
                    NewPage();
                CurrentPage->PutText(left, top, text, font);
            }
        }

        void TXPMiddleEnd::AddText(const std::wstring& text, TXPFontSettings* font)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::AddText\n";

            if(font == NULL)
                font = DefaultFont;
            if(CurrentPage == NULL)
                NewPage();

            // static optimizes the output, by concatenating it
            // #define STATIC
            #ifdef STATIC
            static
            #endif
            std::wstring foo;
            #ifdef STATIC
            static
            #endif
            float initial_left_cm = left_cm;
            for(std::wstring::const_iterator i = text.begin(); i != text.end(); i++)
            {
                switch(*i)
                {
                    default:
                        foo += *i;
                        left_cm += (0.41f/20.0f) * font->FontSize;
                        if(left_cm < CurrentPage->PageWidth_cm - CurrentPage->MarginRight_cm)
                            break; // otherwise, we use the next case to generate a linebreak
                    case '\n':
                        if(foo != L"")
                            AddText(initial_left_cm, top_cm, foo);
                        foo = L"";
                        top_cm += (0.52f/20.0f) * font->FontSize;
                        if(top_cm > CurrentPage->PageHeight_cm - CurrentPage->MarginBottom_cm)
                            NewPage();
                        left_cm = 0.0f + CurrentPage->MarginLeft_cm;
                        initial_left_cm = left_cm;
                        break;
                }
            }

            #ifndef STATIC
            if(foo != L"")
                AddText(initial_left_cm, top_cm, foo, font);
            #endif
        }



        void TXPMiddleEnd::NewPage()
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::NewPage\n";
            NewPage(DefaultWidth_cm, DefaultHeight_cm, DefaultMarginTop_cm, DefaultMarginLeft_cm, DefaultMarginRight_cm, DefaultMarginBottom_cm, DefaultResolution_dpi);
        }
        void TXPMiddleEnd::NewPage(float Width, float Height)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::NewPage\n";
            NewPage(Width, Height, DefaultMarginTop_cm, DefaultMarginLeft_cm, DefaultMarginRight_cm, DefaultMarginBottom_cm, DefaultResolution_dpi);
        }
        void TXPMiddleEnd::NewPage(float MarginTop, float MarginLeft, float MarginRight, float MarginBottom)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::NewPage\n";
            NewPage(DefaultWidth_cm, DefaultHeight_cm, MarginTop, MarginLeft, MarginRight, MarginBottom, DefaultResolution_dpi);
        }
        void TXPMiddleEnd::NewPage(float Width, float Height, float MarginTop, float MarginLeft, float MarginRight, float MarginBottom)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::NewPage\n";
            NewPage(Width, Height, MarginTop, MarginLeft, MarginRight, MarginBottom, DefaultResolution_dpi);
        }
        void TXPMiddleEnd::NewPage(float Width, float Height, float MarginTop, float MarginLeft, float MarginRight, float MarginBottom, float Resolution_dpi)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::NewPage\n";
            CurrentPage = new AbstractPage(NextPageNumber, Width, Height, MarginTop, MarginLeft, MarginRight, MarginBottom, Resolution_dpi);
            NextPageNumber++;
            Pages.push_back(CurrentPage);

            left_cm = 0.0f + CurrentPage->MarginLeft_cm;
            top_cm = 0.52f + CurrentPage->MarginTop_cm;
        }


        void TXPMiddleEnd::AddLabel(const std::wstring& label)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::AddLabel\n";
            if(CurrentPage == NULL)
                NewPage();
            PageLinks[label] = CurrentPage;
        }
        int TXPMiddleEnd::GetPageNumber(const std::wstring& label)
        {
            if(PageLinks.find(label) == PageLinks.end())
                return -1;
            return PageLinks[label]->PageNumber;
        }
        void TXPMiddleEnd::AddPageLink( float left, float top, float width, float height, const std::wstring& label)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::AddPageLink\n";
            if(CurrentPage == NULL)
                NewPage();
            CurrentPage->PutPageLink(left, top, width, height, label);
        }
        void TXPMiddleEnd::AddHyperlink( float left, float top, float width, float height, const std::wstring& url)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::AddHyperlink\n";
            if(CurrentPage == NULL)
                NewPage();
            CurrentPage->PutHyperlink(left, top, width, height, url);
        }
        void TXPMiddleEnd::AddBookmark( int depthRelativeToPrevious, const std::wstring& Caption)
        {
            if(DEBUG_EXECUTION) cerr << "TXPMiddleEnd::AddBookmark\n";
            if(CurrentPage == NULL)
                NewPage();
            CurrentPage->PutBookmark(depthRelativeToPrevious, Caption);
        }

        void TXPMiddleEnd::ProduceResult(TXPBackend* backend)
        {
            if(DEBUG_EXPORT) cerr << "TXPMiddleEnd::ProduceResult\n";

            int page = 1;
            for(std::list<AbstractPage*>::iterator p = Pages.begin(); p != Pages.end(); p++)
                (*p)->PageNumber = page++;

            backend->Initialize(Pages.size(), CreationTime, Author, Title);
            for(std::list<AbstractPage*>::iterator p = Pages.begin(); p != Pages.end(); p++)
                (*p)->WriteToBackend(this, backend);
            backend->Finish();
        }










