%{
#include "../../include/txp/txp.h"
#include "txp_parser.hpp"
#include "../../include/parser/txp_lexer.h"
#define yyterminate() return yy::TXPParser::token::END

wchar_t UTF8toWCHAR(char* utf8string, int numbytes);
extern bool LEXER_DEBUG;
%}

%option c++
%option 8bit
%option case-sensitive
%option yyclass="TXPLexer"
%option full
%option noyywrap
%option nounput
%option batch
%option yylineno
%option prefix="TXP_"


%x dump
%x code
%%
%{
    static bool firstrun = true;
    if(firstrun)
    {
        BEGIN(dump);
        if(LEXER_DEBUG) cerr << "flex: BEGIN(dump)\n";
        firstrun = false;
    }
%}


<*>"//"[^\n]*\n                               { /* ignore comments */ }

<*>[\ \n\t\r]*"{"[\ \n\t\r]*                  { if(LEXER_DEBUG) cerr << "flex: OPENBRACES\n";  return yy::TXPParser::token::OPENBRACES;}
<*>[\ \n\t\r]*"}"[\ \n\t\r]*                  { if(LEXER_DEBUG) cerr << "flex: CLOSEBRACES\n"; return yy::TXPParser::token::CLOSEBRACES;}

<dump>[\ \n\t\r]+                             { if(LEXER_DEBUG) cerr << "flex: CHAR ' '\n";   (*yylval).stringval = new(std::wstring)(L" ");    return yy::TXPParser::token::CHAR;}
<code>[\ \n\t\r]+                             { /* ignore whitespace within code-regions */ }

<*>[\ \n\t\r]*"\\\\"[\ \n\t\r]*               { if(LEXER_DEBUG) cerr << "flex: CHAR '\\n'\n"; (*yylval).stringval = new(std::wstring)(L"\n");   return yy::TXPParser::token::CHAR;}
<*>"\\\""                                     { if(LEXER_DEBUG) cerr << "flex: CHAR '\\\"'\n"; (*yylval).stringval = new(std::wstring)(L"\"");  return yy::TXPParser::token::CHAR;}
<*>"\\{"                                      { if(LEXER_DEBUG) cerr << "flex: CHAR '{'\n";   (*yylval).stringval = new(std::wstring)(L"{");    return yy::TXPParser::token::CHAR;}
<*>"\\}"                                      { if(LEXER_DEBUG) cerr << "flex: CHAR '}'\n";   (*yylval).stringval = new(std::wstring)(L"}");    return yy::TXPParser::token::CHAR;}

<code>"unicode"                               { if(LEXER_DEBUG) cerr << "flex: UNICODE\n";     return yy::TXPParser::token::UNICODE; }

<*>[\ \n\t\r]*"\\"                            { if(LEXER_DEBUG) cerr << "yy_push_state(code)\n"; yy_push_state(code);}


<code>"("                                     { if(LEXER_DEBUG) cerr << "flex: OPEN\n";      return yy::TXPParser::token::OPEN; }
<code>")"[\n\t\r]*                            { if(LEXER_DEBUG) cerr << "flex: CLOSE\n";     return yy::TXPParser::token::CLOSE; }
<code>";"                                     { if(LEXER_DEBUG) cerr << "flex: SEMICOLON\n"; return yy::TXPParser::token::SEMICOLON; }
<code>","                                     { if(LEXER_DEBUG) cerr << "flex: COMMA\n";     return yy::TXPParser::token::COMMA; }

<code>"for"                                   { if(LEXER_DEBUG) cerr << "flex: FOR\n";       return yy::TXPParser::token::FOR; }
<code>"newpage"                               { if(LEXER_DEBUG) cerr << "flex: NEWPAGE\n";   return yy::TXPParser::token::NEWPAGE; }
<code>"label"                                 { if(LEXER_DEBUG) cerr << "flex: LABEL\n";     return yy::TXPParser::token::LABEL; }
<code>"pagelink"                              { if(LEXER_DEBUG) cerr << "flex: PAGELINK\n";  return yy::TXPParser::token::PAGELINK; }
<code>"hyperlink"                             { if(LEXER_DEBUG) cerr << "flex: HYPERLINK\n"; return yy::TXPParser::token::HYPERLINK; }
<code>"bookmark"                              { if(LEXER_DEBUG) cerr << "flex: BOOKMARK\n";  return yy::TXPParser::token::BOOKMARK; }
<code>"if"                                    { if(LEXER_DEBUG) cerr << "flex: IF\n";        return yy::TXPParser::token::IF; }
<code>"else"                                  { if(LEXER_DEBUG) cerr << "flex: ELSE\n";      return yy::TXPParser::token::ELSE; }
<code>"true"                                  { if(LEXER_DEBUG) cerr << "flex: TRUE\n";      return yy::TXPParser::token::TRUE; }
<code>"false"                                 { if(LEXER_DEBUG) cerr << "flex: FALSE\n";     return yy::TXPParser::token::FALSE; }
<code>"void"                                  { if(LEXER_DEBUG) cerr << "flex: VOID\n";      return yy::TXPParser::token::VOID; }

<code>"&&"                                    { if(LEXER_DEBUG) cerr << "flex: AND\n";       return yy::TXPParser::token::AND; }
<code>"||"                                    { if(LEXER_DEBUG) cerr << "flex: OR\n";        return yy::TXPParser::token::OR; }
<code>"!"                                     { if(LEXER_DEBUG) cerr << "flex: NOT\n";       return yy::TXPParser::token::NOT; }

<code>"="                                     { if(LEXER_DEBUG) cerr << "flex: ASSIGN\n";    return yy::TXPParser::token::ASSIGN; }
<code>"=="                                    { if(LEXER_DEBUG) cerr << "flex: EQ\n";        return yy::TXPParser::token::EQ; }
<code>"!="                                    { if(LEXER_DEBUG) cerr << "flex: NEQ\n";       return yy::TXPParser::token::NEQ; }
<code>"<="                                    { if(LEXER_DEBUG) cerr << "flex: LE\n";        return yy::TXPParser::token::LE; }
<code>"<"                                     { if(LEXER_DEBUG) cerr << "flex: LT\n";        return yy::TXPParser::token::LT; }
<code>">"                                     { if(LEXER_DEBUG) cerr << "flex: GT\n";        return yy::TXPParser::token::GT; }
<code>">="                                    { if(LEXER_DEBUG) cerr << "flex: GE\n";        return yy::TXPParser::token::GE; }

<code>"+"                                     { if(LEXER_DEBUG) cerr << "flex: ADD\n";       return yy::TXPParser::token::ADD; }
<code>"-"                                     { if(LEXER_DEBUG) cerr << "flex: SUB\n";       return yy::TXPParser::token::SUB; }
<code>"*"                                     { if(LEXER_DEBUG) cerr << "flex: MUL\n";       return yy::TXPParser::token::MUL; }
<code>"/"                                     { if(LEXER_DEBUG) cerr << "flex: DIV\n";       return yy::TXPParser::token::DIV; }
<code>"%"                                     { if(LEXER_DEBUG) cerr << "flex: MOD\n";       return yy::TXPParser::token::MOD; }
<code>"+="                                    { if(LEXER_DEBUG) cerr << "flex: ADDASSIGN\n"; return yy::TXPParser::token::ADDASSIGN; }
<code>"-="                                    { if(LEXER_DEBUG) cerr << "flex: SUBASSIGN\n"; return yy::TXPParser::token::SUBASSIGN; }
<code>"*="                                    { if(LEXER_DEBUG) cerr << "flex: MULASSIGN\n"; return yy::TXPParser::token::MULASSIGN; }
<code>"/="                                    { if(LEXER_DEBUG) cerr << "flex: DIVASSIGN\n"; return yy::TXPParser::token::DIVASSIGN; }
<code>"%="                                    { if(LEXER_DEBUG) cerr << "flex: MODASSIGN\n"; return yy::TXPParser::token::MODASSIGN; }

<code>[0-9]+                                  { if(LEXER_DEBUG) cerr << "flex: INT\n";       (*yylval).intval = new(int)(atoi(yytext));   return yy::TXPParser::token::INT; }
<code>[0-9]*\.[0-9]+                          { if(LEXER_DEBUG) cerr << "flex: FLOAT\n";     (*yylval).floatval = new(float)(atof(yytext)); return yy::TXPParser::token::FLOAT; }
<code>[a-zA-Z\:][0-9a-zA-Z\:]*                { if(LEXER_DEBUG) cerr << "flex: IDENTIFIER\n";
                                                std::wstring foo;
                                                for(int i = 0; i < yyleng; i++)
                                                    foo += (wchar_t)(yytext[i]);
                                                (*yylval).stringval = new(std::wstring)(foo);
                                                return yy::TXPParser::token::IDENTIFIER; }
<code>\"[^\"]*\"                              { if(LEXER_DEBUG) cerr << "flex: IDENTIFIER\n";
                                                std::string foo(yytext);
                                                (*yylval).stringval = new(std::wstring)(Utf8StringToWString(foo.substr(1, foo.size()-2)));
                                                return yy::TXPParser::token::IDENTIFIER; }


<dump>[\x00-\x7f]                              { if(LEXER_DEBUG) cerr << "flex: CHAR\n";      (*yylval).stringval = new(std::wstring)(1,UTF8toWCHAR(yytext,1)); return yy::TXPParser::token::CHAR; }
<dump>[\xc2-\xdf][\x80-\xbf]                   { if(LEXER_DEBUG) cerr << "flex: CHAR2\n";     (*yylval).stringval = new(std::wstring)(1,UTF8toWCHAR(yytext,2)); return yy::TXPParser::token::CHAR; }
<dump>[\xe0-\xef][\x80-\xbf][\x80-\xbf]        { if(LEXER_DEBUG) cerr << "flex: CHAR3\n";     (*yylval).stringval = new(std::wstring)(1,UTF8toWCHAR(yytext,3)); return yy::TXPParser::token::CHAR; }
<dump>[\xf0-\xf4][\x80-\xbf][\x80-\xbf][\x80-\xbf]  { if(LEXER_DEBUG) cerr << "flex: CHAR4\n";     (*yylval).stringval = new(std::wstring)(1,UTF8toWCHAR(yytext,4)); return yy::TXPParser::token::CHAR; }
<dump>.                                        { cerr << "lexical error: unexpected byte " << (int)(yytext[0]) << " (is your file UTF-8 encoded?)\n"; }

%%


TXPLexer::TXPLexer(std::istream& is) : TXP_FlexLexer(&is)
{
}

int TXPLexer::yylex(yy::TXPParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}

int TXPLexer::GetCurrentLineNumber()
{
    return yylineno;
}

void TXPLexer::pop_state()
{
    if(LEXER_DEBUG) cerr << "yy_pop_state()\n";
    yy_pop_state();
}

void TXPLexer::push_state_code()
{
    if(LEXER_DEBUG) cerr << "yy_push_state(code)\n";
    yy_push_state(code);
}

